/*
	Genome-wide Efficient Mixed Model Association (GEMMA)
    Copyright (C) 2011  Xiang Zhou and Matthew Stephens

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include "gsl/gsl_vector.h"
#include "gsl/gsl_matrix.h"

using namespace std;

extern "C" void dsyev_(char* JOBZ, char* UPLO, int *N, double *A, int *LDA, double *W, double *WORK, int *LWORK, int *INFO);


//eigen value decomposition, matrix A is destroyed
void lapack_eigen_symmv (gsl_matrix *A, gsl_vector *eval, gsl_matrix *evec)
{
	int N=A->size1, LDA=A->size1, INFO, LWORK=-1;
	char JOBZ='V', UPLO='L';
	double temp[1];

	if (N!=(int)A->size2 || N!=(int)eval->size) {cout<<"Matrix needs to be symmetric and same dimension in lapack_eigen_symmv."<<endl; return;}

	dsyev_(&JOBZ, &UPLO, &N, A->data, &LDA, eval->data, temp, &LWORK, &INFO);
	if (INFO!=0) {cout<<"Work space estimate unsuccessful in lapack_eigen_symmv."<<endl; return;}
	
	LWORK=(int)temp[0];
	double *WORK=new double [LWORK];	
	dsyev_(&JOBZ, &UPLO, &N, A->data, &LDA, eval->data, WORK, &LWORK, &INFO);
	if (INFO!=0) {cout<<"Eigen decomposition unsuccessful in lapack_eigen_symmv."<<endl; return;}
	
	gsl_matrix_view A_sub=gsl_matrix_submatrix(A, 0, 0, N, N);
	gsl_matrix_memcpy (evec, &A_sub.matrix);
	gsl_matrix_transpose (evec);

  	delete [] WORK;
	return;
}





